/// <reference types="node" />
import { EventEmitter } from 'events';
import { ConnectOptions } from './lib/interfaces';
import { ArmingStatusReport, OutputStatusReport, TextStringDescriptionReport, ZoneDefinitionReport, ZonePartitionReport, ZoneStatusReport } from './lib/messages';
import { ArmMode, TextDescriptionType } from './lib/enums';
declare class Elk extends EventEmitter {
    port: number;
    host: string;
    options: ConnectOptions;
    private isAuthorized;
    private connection;
    constructor(port?: number, host?: string, options?: ConnectOptions);
    connect(): void;
    onConnect(): void;
    onDataReceived(data: any): any;
    /**
     * Arm an Area by id.
     *
     * @param {number} areaId
     * @param {ArmMode} armMode
     * @param {string} keypadCode
     */
    arm(areaId: number, armMode: ArmMode, keypadCode: string): void;
    /**
     * Disarm an Area by id.
     *
     * @param {number} areaId
     * @param {string} keypadCode
     */
    disarm(areaId: number, keypadCode: string): void;
    /**
     * Activates a task. The task is deactivated when it completes.
     *
     * @param {number} taskId
     */
    activateTask(taskId: number): void;
    /**
     * Turn output on.
     *
     * @param {number} outputId
     * @param {number} seconds - Number of seconds output will be active
     */
    setOutputOn(outputId: number, seconds: number): void;
    /**
     * Turn output off.
     *
     * @param {number} outputId
     */
    setOutputOff(outputId: number): void;
    /**
     * Command Elk panel to speak a message over it's speaker.
     *
     * @param {string} message
     */
    speak(message: string): void;
    /**
     * Toggles a control Output On/Off.
     *
     * @param {number} outputId
     */
    toggleOutput(outputId: number): void;
    /**
     * Request the Control Output Status report from Elk panel.
     *
     * @param {number} [timeout=5000]
     * @returns
     */
    requestOutputStatusReport(timeout?: number): Promise<OutputStatusReport>;
    /**
     * Bypass a Zone.
     *
     * @param {number} zoneId
     * @param {number} areaId
     * @param {string} keypadCode
     */
    bypassZone(zoneId: number, areaId: number, keypadCode: string): void;
    /**
     * Requests an Arming Status Report from the Elk panel.
     *
     * @param {number} [timeout=5000]
     * @returns {ArmingStatusReport}
     */
    requestArmingStatus(timeout?: number): Promise<ArmingStatusReport>;
    /**
     * Request Keypad Area assignments.
     *
     * @param {number} [timeout=5000]
     * @returns {Promise<number[]>}
     *
     * @memberOf Elk
     */
    requestAreas(timeout?: number): Promise<number[]>;
    /**
     * Requests a Zone Definition Report from the Elk panel.
     *
     * @param {number} [timeout=5000]
     * @returns {ZoneDefinitionReport}
     */
    requestZoneDefinitionReport(timeout?: number): Promise<ZoneDefinitionReport>;
    /**
     * Requests a Zone Partition Report from the Elk panel.
     *
     * @param {number} [timeout=5000]
     * @returns {ZonePartitionReport}
     */
    requestZonePartitionReport(timeout?: number): Promise<ZonePartitionReport>;
    /**
     * Requests a Zone Status Report from the Elk panel.
     *
     * @param {number} [timeout=5000]
     * @returns {ZoneStatusReport}
     */
    requestZoneStatusReport(timeout?: number): Promise<ZoneStatusReport>;
    /**
     * Request zone voltage report
     *
     * @param {number} id
     * @param {number} [timeout=5000]
     * @returns {Promise<TextStringDescriptionReport>}
     *
     * @memberOf Elk
     */
    requestZoneVoltageReport(id: number, timeout?: number): Promise<unknown>;
    /**
     * Request text description
     *
     * @param {number} id
     * @param {TextDescriptionType} type
     * @param {number} [timeout=5000]
     * @returns {Promise<TextStringDescriptionReport>}
     *
     * @memberOf Elk
     */
    requestTextDescription(id: number, type: TextDescriptionType, timeout?: number): Promise<TextStringDescriptionReport>;
    /**
     * For internal use by requestTextDescriptionAll. Asynchronous call to retreived
     * a Text Description.
     * @param {number} id
     * @param {TextDescriptionType} type
     * @param {any} cb
     * @param {number} [timeout=15000]
     */
    getDescription(id: number, type: TextDescriptionType, cb: any, timeout?: number): void;
    /**
     * Method returns the configured Text Descriptions, by type. We start at id=1.
     * If the requested id is not configured, Elk will return the next configured item (to speed up requests)
     * or 000 if no more valid names are found. Consult RS232 protocol guide for more info on SD messages.
     * So we'll recursively query the panel until we reach the max length, by type, or receive 000.
     * NOTE: This can be used to determine what has been configured on panel since a description is not returned
     * for unconfigured items. Useful for Tasks, Outputs, etc that don't have an Api call for retreving
     * configuration/definition.
     * @param {string} type
     * @param {number} [timeout=15000]
     * @returns {TextStringDescriptionReport}
     */
    requestTextDescriptionAll(type: TextDescriptionType, timeout?: number): Promise<TextStringDescriptionReport[]>;
    /**
     * Set thermostat data.
     *
     * @param {number} thermostatId - The thermostat number to program (1-16).
     * @param {number} value - The value to set (00-99).
     * @param {number} element - The element to set.
     */
    setThermostat(thermostatId: number, value: number, element: number): void;
    /**
     * Disconnects from the Elk M1XEP.
     */
    disconnect(): void;
}
export = Elk;
