"use strict";
const tls_1 = require("tls");
const net_1 = require("net");
const events_1 = require("events");
const factory_1 = require("./lib/factory");
const messages_1 = require("./lib/messages");
const enums_1 = require("./lib/enums");
const types_1 = require("./lib/types");
const utils_1 = require("./lib/utils");
class Elk extends events_1.EventEmitter {
    constructor(port = 2101, host = '192.168.1.0', options = {
        secure: false,
        rejectUnauthorized: false,
        secureProtocol: 'TLSv1_method'
    }) {
        super();
        this.port = port;
        this.host = host;
        this.options = options;
        this.isAuthorized = false;
        this.connection = null;
    }
    connect() {
        if (this.options.secure) {
            this.connection = tls_1.connect(this.port, this.host, {
                rejectUnauthorized: this.options.rejectUnauthorized,
                secureProtocol: this.options.secureProtocol
            }, () => this.onConnect());
        }
        else {
            this.connection = net_1.connect({
                host: this.host,
                port: this.port
            }, () => this.onConnect());
        }
        this.connection.setEncoding('ascii');
        // Listen for incoming data 
        this.connection.on('data', (data) => this.onDataReceived(data));
        // error event handler
        this.connection.on('error', (err) => {
            if (err.code === 'ECONNREFUSED') {
                this.emit('error', 'Connection to M1XEP failed!');
            }
            else if (err.code === 'ECONNRESET') {
                this.emit('error', err.code);
                // connection was reset, attempt to reconnect
                if (this.connection) {
                    this.connection.destroy();
                }
                this.connect();
            }
            else {
                this.emit('error', err.code);
            }
        });
        // close event handler
        this.connection.on('close', () => {
            this.emit('end', 'The connection to the Elk M1 has been lost');
        });
    }
    onConnect() {
        this.emit('connected');
    }
    onDataReceived(data) {
        // check for elk auth requests
        if (data == 'Username:') {
            return this.connection.write(this.options.userName + '\r\n');
        }
        else if (data.indexOf('Password:') != -1) {
            return this.connection.write(this.options.password + '\r\n');
        }
        else if (data.indexOf('Elk-M1XEP: Login successful.') !== -1) {
            this.isAuthorized = true;
            this.emit('connected');
            return;
        }
        // we are getting a weird message during auth process
        if (this.options.secure && (!this.isAuthorized || data.substring(0, 2) === '**')) {
            return;
        }
        try {
            // Split on newline in case multiple messages are received at one time.
            let messages = data.trim().split('\n');
            messages.forEach((message, index) => {
                // Remove carriage return
                message = message.replace('\r', '');
                // Parse message using Elk factory method
                let elkMessage = factory_1.default(message);
                // Emit message
                this.emit('*', elkMessage);
                this.emit(elkMessage.type, elkMessage);
            });
        }
        catch (e) {
            this.emit('error', e);
        }
    }
    /**
     * Arm an Area by id.
     *
     * @param {number} areaId
     * @param {ArmMode} armMode
     * @param {string} keypadCode
     */
    arm(areaId, armMode, keypadCode) {
        let elk = new messages_1.ElkMessage(`a${armMode}${areaId}${utils_1.leftPad(keypadCode, 6, '0')}`, null);
        this.connection.write(`${elk.message}\r\n`);
    }
    /**
     * Disarm an Area by id.
     *
     * @param {number} areaId
     * @param {string} keypadCode
     */
    disarm(areaId, keypadCode) {
        let elk = new messages_1.ElkMessage(`a0${areaId}${utils_1.leftPad(keypadCode, 6, '0')}`, null);
        this.connection.write(`${elk.message}\r\n`);
    }
    /**
     * Activates a task. The task is deactivated when it completes.
     *
     * @param {number} taskId
     */
    activateTask(taskId) {
        let elk = new messages_1.ElkMessage(`tn${utils_1.leftPad(taskId.toString(), 3, '0')}`, null);
        this.connection.write(`${elk.message}\r\n`);
    }
    /**
     * Turn output on.
     *
     * @param {number} outputId
     * @param {number} seconds - Number of seconds output will be active
     */
    setOutputOn(outputId, seconds) {
        let elk = new messages_1.ElkMessage(`cn${utils_1.leftPad(outputId.toString(), 3, '0')}${utils_1.leftPad(seconds.toString(), 5, '0')}`, null);
        this.connection.write(`${elk.message}\r\n`);
    }
    /**
     * Turn output off.
     *
     * @param {number} outputId
     */
    setOutputOff(outputId) {
        let elk = new messages_1.ElkMessage(`cf${utils_1.leftPad(outputId.toString(), 3, '0')}`, null);
        this.connection.write(`${elk.message}\r\n`);
    }
    /**
     * Command Elk panel to speak a message over it's speaker.
     *
     * @param {string} message
     */
    speak(message) {
        let words = message.split(' ');
        words.forEach(word => {
            let index = enums_1.Words[word.toLowerCase()];
            let elk = new messages_1.ElkMessage(`sw${utils_1.leftPad(index, 3, '0')}`, null);
            this.connection.write(`${elk.message}\r\n`);
        });
    }
    /**
     * Toggles a control Output On/Off.
     *
     * @param {number} outputId
     */
    toggleOutput(outputId) {
        let elk = new messages_1.ElkMessage(`ct${utils_1.leftPad(outputId.toString(), 3, '0')}`, null);
        this.connection.write(`${elk.message}\r\n`);
    }
    /**
     * Request the Control Output Status report from Elk panel.
     *
     * @param {number} [timeout=5000]
     * @returns
     */
    requestOutputStatusReport(timeout = 5000) {
        return new Promise((resolve, reject) => {
            //Listen for response
            this.once('CS', (response) => {
                resolve(response);
            });
            //Send the command
            let elk = new messages_1.ElkMessage('cs', null);
            this.connection.write(`${elk.message}\r\n`);
            //Setup timeout
            setTimeout(function () {
                reject('Timout occured before Control Output Status (cs) was received.');
            }, timeout);
        });
    }
    /**
     * Bypass a Zone.
     *
     * @param {number} zoneId
     * @param {number} areaId
     * @param {string} keypadCode
     */
    bypassZone(zoneId, areaId, keypadCode) {
        if (zoneId === null) {
            throw new TypeError('Zone id is a required option');
        }
        let elk = new messages_1.ElkMessage(`zb${utils_1.leftPad(zoneId.toString(), 3, '0')}${areaId}${utils_1.leftPad(keypadCode, 6, '0')}`, null);
        this.connection.write(`${elk.message}\r\n`);
    }
    /**
     * Requests an Arming Status Report from the Elk panel.
     *
     * @param {number} [timeout=5000]
     * @returns {ArmingStatusReport}
     */
    requestArmingStatus(timeout = 5000) {
        return new Promise((resolve, reject) => {
            //Listen for response
            this.once('AS', (response) => {
                resolve(response);
            });
            //Send the command
            let elk = new messages_1.ElkMessage('as', null);
            this.connection.write(`${elk.message}\r\n`);
            //Setup timeout
            setTimeout(function () {
                reject('Timout occured before Arming Status (as) was received.');
            }, timeout);
        });
    }
    /**
     * Request Keypad Area assignments.
     *
     * @param {number} [timeout=5000]
     * @returns {Promise<number[]>}
     *
     * @memberOf Elk
     */
    requestAreas(timeout = 5000) {
        return new Promise((resolve, reject) => {
            //Listen for response
            this.once('KA', (response) => {
                resolve(response);
            });
            //Send the command
            let elk = new messages_1.ElkMessage('ka', null);
            this.connection.write(`${elk.message}\r\n`);
            //Setup timeout
            setTimeout(function () {
                reject('Timout occured before Area request (ka) was received.');
            }, timeout);
        });
    }
    /**
     * Requests a Zone Definition Report from the Elk panel.
     *
     * @param {number} [timeout=5000]
     * @returns {ZoneDefinitionReport}
     */
    requestZoneDefinitionReport(timeout = 5000) {
        return new Promise((resolve, reject) => {
            //Listen for response
            this.once('ZD', (response) => {
                resolve(response);
            });
            //Send the command
            let elk = new messages_1.ElkMessage('zd', null);
            this.connection.write(`${elk.message}\r\n`);
            //Setup timeout
            setTimeout(function () {
                reject('Timout occured before Zone Definition (zd) was received.');
            }, timeout);
        });
    }
    /**
     * Requests a Zone Partition Report from the Elk panel.
     *
     * @param {number} [timeout=5000]
     * @returns {ZonePartitionReport}
     */
    requestZonePartitionReport(timeout = 5000) {
        return new Promise((resolve, reject) => {
            //Listen for response
            this.once('ZP', (response) => {
                resolve(response);
            });
            //Send the command
            let elk = new messages_1.ElkMessage('zp', null);
            this.connection.write(`${elk.message}\r\n`);
            //Setup timeout
            setTimeout(function () {
                reject('Timout occured before Zone Partition (zp) was received.');
            }, timeout);
        });
    }
    /**
     * Requests a Zone Status Report from the Elk panel.
     *
     * @param {number} [timeout=5000]
     * @returns {ZoneStatusReport}
     */
    requestZoneStatusReport(timeout = 5000) {
        return new Promise((resolve, reject) => {
            //Listen for response
            this.once('ZS', (response) => {
                resolve(response);
            });
            //Send the command
            let elk = new messages_1.ElkMessage('zs', null);
            this.connection.write(`${elk.message}\r\n`);
            //Setup timeout
            setTimeout(function () {
                reject('Timout occured before Zone Status Report (zs) was received.');
            }, timeout);
        });
    }
    /**
     * Request zone voltage report
     *
     * @param {number} id
     * @param {number} [timeout=5000]
     * @returns {Promise<TextStringDescriptionReport>}
     *
     * @memberOf Elk
     */
    requestZoneVoltageReport(id, timeout = 5000) {
        return new Promise((resolve, reject) => {
            //Listen for response
            this.once('ZV', (response) => {
                resolve(response);
            });
            //Send the command
            let elk = new messages_1.ElkMessage(`zv${utils_1.leftPad(id.toString(), 3, 0)}`, null);
            this.connection.write(`${elk.message}\r\n`);
            //Setup timeout
            setTimeout(function () {
                reject('Timout occured before Zone Voltage Report (zv) was received.');
            }, timeout);
        });
    }
    /**
     * Request text description
     *
     * @param {number} id
     * @param {TextDescriptionType} type
     * @param {number} [timeout=5000]
     * @returns {Promise<TextStringDescriptionReport>}
     *
     * @memberOf Elk
     */
    requestTextDescription(id, type, timeout = 5000) {
        return new Promise((resolve, reject) => {
            //Listen for response
            this.once('SD', (response) => {
                resolve(response);
            });
            //Send the command
            let elk = new messages_1.ElkMessage(`sd${utils_1.leftPad(type.toString(), 2, 0)}${utils_1.leftPad(id.toString(), 3, 0)}`, null);
            this.connection.write(`${elk.message}\r\n`);
            //Setup timeout
            setTimeout(function () {
                reject('Timout occured before Text Description (sd) was received.');
            }, timeout);
        });
    }
    /**
     * For internal use by requestTextDescriptionAll. Asynchronous call to retreived
     * a Text Description.
     * @param {number} id
     * @param {TextDescriptionType} type
     * @param {any} cb
     * @param {number} [timeout=15000]
     */
    getDescription(id, type, cb, timeout = 15000) {
        //Listen for response
        this.once('SD', (response) => {
            cb(response);
        });
        //Send the command
        let elk = new messages_1.ElkMessage(`sd${utils_1.leftPad(type.toString(), 2, 0)}${utils_1.leftPad(id.toString(), 3, 0)}`, null);
        this.connection.write(`${elk.message}\r\n`);
        //Setup timeout
        setTimeout(function () {
            cb({ error: 'Timout occured before Text Description (sd) was received.' });
        }, timeout);
    }
    /**
     * Method returns the configured Text Descriptions, by type. We start at id=1.
     * If the requested id is not configured, Elk will return the next configured item (to speed up requests)
     * or 000 if no more valid names are found. Consult RS232 protocol guide for more info on SD messages.
     * So we'll recursively query the panel until we reach the max length, by type, or receive 000.
     * NOTE: This can be used to determine what has been configured on panel since a description is not returned
     * for unconfigured items. Useful for Tasks, Outputs, etc that don't have an Api call for retreving
     * configuration/definition.
     * @param {string} type
     * @param {number} [timeout=15000]
     * @returns {TextStringDescriptionReport}
     */
    requestTextDescriptionAll(type, timeout = 15000) {
        // Text Description items received from panel
        const items = [];
        const recursiveCall = (id, resolve, reject) => {
            this.getDescription(id, type, (data) => {
                if (data.error) {
                    reject(data.error);
                }
                // If the returned id == 0, then no more configured items.
                if (data.id > 0 && id < types_1.textDescriptionMaxRange[enums_1.TextDescriptionType[type]]) {
                    //panel returned a text description
                    items.push(data);
                    // Since panel returns the 'next' configured item, we need set the id to 
                    // the last retreived one.
                    id = data.id;
                    id++; // Increment to next item
                    recursiveCall(id, resolve, reject);
                }
                else {
                    // Return items to caller
                    resolve(items);
                }
            }, timeout);
        };
        // Promise is resolved when all items have been received.
        return new Promise((resolve, reject) => {
            recursiveCall(1, resolve, reject);
        });
    }
    /**
     * Set thermostat data.
     *
     * @param {number} thermostatId - The thermostat number to program (1-16).
     * @param {number} value - The value to set (00-99).
     * @param {number} element - The element to set.
     */
    setThermostat(thermostatId, value, element) {
        const message = 'The value parameter is outside accepted range.';
        // Do some validation
        if (element < 0 || element > 5)
            throw new Error(message.replace('value', 'element'));
        switch (element) {
            case 0:
                if (value < 0 || value > 4)
                    throw new Error(message);
                break;
            case 1:
                if (value < 0 || value > 1)
                    throw new Error(message);
                break;
            case 2:
                if (value < 0 || value > 1)
                    throw new Error(message);
                break;
            case 4:
                if (value < 1 || value > 99)
                    throw new Error(message);
                break;
            case 5:
                if (value < 1 || value > 99)
                    throw new Error(message);
                break;
            default:
        }
        let elk = new messages_1.ElkMessage(`ts${utils_1.leftPad(thermostatId.toString(), 2, '0')}${utils_1.leftPad(value.toString(), 2, '0')}${element.toString()}`, null);
        this.connection.write(`${elk.message}\r\n`);
    }
    /**
     * Disconnects from the Elk M1XEP.
     */
    disconnect() {
        if (this.connection) {
            this.connection.destroy();
        }
    }
}
module.exports = Elk;
