import { AreaReport, ZoneDefinition } from './models';
/**
 * Base type for all response/messages. Responsible for creating new Commands to send to Elk M1 OR
 * parsing/validating a response from Elk.
 * @export
 * @class ElkMessage
 */
export declare class ElkMessage {
    message: string;
    protected body: string;
    private type;
    private hexLength;
    private checkSum;
    /**
     * Creates an instance of ElkMessage.
     *
     * @param {string} command - Instantiate message from a new command, i.e. command = 'zs'.
     * @param {string} response - Instantiate message from an Elk response, i.e. response = '0AZC002200CE'.
     */
    constructor(command: string, response: string);
    validChecksum(): boolean;
    calcChecksum(): string;
    /**
     * Convert to 2 byte hex value.
     *
     * @param {number} i
     * @returns
     */
    hex2(i: number): string;
}
/**
 * Represents an Area Change Update message (AS)
 *
 * @export
 * @class ArmingStatusReport
 * @extends {ElkMessage}
 */
export declare class ArmingStatusReport extends ElkMessage {
    areas: AreaReport[];
    constructor(response: string);
}
/**
 * Represents Entry/Exit Time Data (EE)
 *
 * @export
 * @class EntryExitTime
 * @extends {ElkMessage}
 */
export declare class EntryExitTime extends ElkMessage {
    areaId: number;
    timerType: number;
    timer1: number;
    timer2: number;
    armedState: string;
    constructor(response: string);
}
/**
 * Keypad Areas Report (KA).
 *
 * @export
 * @class KeypadAreasReport
 * @extends {ElkMessage}
 */
export declare class KeypadAreasReport extends ElkMessage {
    /**
     * Distinct areas that are assigned to keypads.
     *
     * @type {number[]}
     */
    areas: number[];
    constructor(response: string);
}
/**
 * Represents System Log Data Update (LD)
 *
 * @export
 * @class LogDataUpdate
 * @extends {ElkMessage}
 */
export declare class LogDataUpdate extends ElkMessage {
    logIndex: string;
    event: string;
    /**
     * Event number data, i.e. Zone number, User number, etc.
     *
     * @type {number}
     */
    id: number;
    areaId: number;
    hour: string;
    minute: string;
    month: string;
    day: number;
    dayOfWeek: string;
    year: string;
    constructor(response: string);
    getEventType(event: number): string;
}
/**
 * Represents an Output Change Update (CC). This message is sent in response to an output
 * status change. Message contains the output number and the new state.
 * @export
 * @class OutputChangeUpdate
 * @extends {ElkMessage}
 */
export declare class OutputChangeUpdate extends ElkMessage {
    id: number;
    state: string;
    constructor(response: string);
}
/**
 * Represents a Control Output Status Report (CS). This message is sent in response to a
 * Control Output Status Request.
 * @export
 * @class OutputStatusReport
 * @extends {ElkMessage}
 */
export declare class OutputStatusReport extends ElkMessage {
    /**
     * Array of output status'. Index 0 = Output 1.
     *
     * @type {string[]}
     */
    outputs: string[];
    constructor(response: string);
}
/**
 * Represents Temerature Reply Data (LW), in reponse to a temperature request.
 *
 * @export
 * @class TemperatureReply
 * @extends {ElkMessage}
 */
export declare class TemperatureReply extends ElkMessage {
    /**
     * Array of keypad temps, 1-16. Index 0 = keypad 1.
     *
     * @type {number[]}
     */
    keypads: number[];
    /**
     * Array of zone sensor temps, 1-16. Index 0 = zone 1.
     *
     * @type {number[]}
     */
    zones: number[];
    constructor(response: string);
}
/**
 * Represents a Thermostat Data Reply (TR).
 *
 * @export
 * @class ThermostatReply
 * @extends {ElkMessage}
 */
export declare class ThermostatReply extends ElkMessage {
    id: string;
    mode: string;
    hold: boolean;
    fan: string;
    temperature: number;
    heatSetPoint: number;
    coolSetPoint: number;
    humidity: string;
    constructor(response: string);
}
/**
 * Represents String Text Description (SD)
 *
 * @export
 * @class TextStringDescriptionReport
 * @extends {ElkMessage}
 */
export declare class TextStringDescriptionReport extends ElkMessage {
    descriptionType: string;
    id: number;
    description: string;
    constructor(response: string);
}
/**
 * Represents a Task Change Update (TC). This message is sent when a task has
 * been activated.
 * @export
 * @class TaskChangeUpdate
 * @extends {ElkMessage}
 */
export declare class TaskChangeUpdate extends ElkMessage {
    taskNumber: number;
    constructor(response: string);
}
/**
 * Represents a Zone Bypass Reply (ZB)
 *
 * @export
 * @class ZoneBypass
 * @extends {ElkMessage}
 */
export declare class ZoneBypass extends ElkMessage {
    id: number;
    bypassed: boolean;
    constructor(response: string);
}
/**
 * Represents a Zone Definition Report (ZD)
 *
 * @export
 * @class ZoneDefinitionReport
 * @extends {ElkMessage}
 */
export declare class ZoneDefinitionReport extends ElkMessage {
    zones: ZoneDefinition[];
    constructor(response: string);
}
/**
 * Represents a Zone Voltage Report (ZV)
 *
 * @export
 * @class ZoneVoltageReport
 * @extends {ElkMessage}
 */
export declare class ZoneVoltageReport extends ElkMessage {
    id: number;
    voltage: number;
    constructor(response: string);
}
/**
 * Represents a Zone Change Update message (ZC).
 *
 * @export
 * @class ZoneChangeUpdate
 * @extends {ElkMessage}
 */
export declare class ZoneChangeUpdate extends ElkMessage {
    id: number;
    physicalStatus: string;
    logicalState: string;
    constructor(response: string);
}
/**
 * Represents Zone Status Report (ZS)
 *
 * @export
 * @class ZoneStatusReport
 */
export declare class ZoneStatusReport extends ElkMessage {
    zones: any;
    constructor(response: string);
}
/**
 * Represents Zone Partition Report (ZP)
 *
 * @export
 * @class ZonePartitionReport
 * @extends {ElkMessage}
 */
export declare class ZonePartitionReport extends ElkMessage {
    zones: any;
    constructor(response: string);
}
