"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ZonePartitionReport = exports.ZoneStatusReport = exports.ZoneChangeUpdate = exports.ZoneVoltageReport = exports.ZoneDefinitionReport = exports.ZoneBypass = exports.TaskChangeUpdate = exports.TextStringDescriptionReport = exports.ThermostatReply = exports.TemperatureReply = exports.OutputStatusReport = exports.OutputChangeUpdate = exports.LogDataUpdate = exports.KeypadAreasReport = exports.EntryExitTime = exports.ArmingStatusReport = exports.ElkMessage = void 0;
const enums_1 = require("./enums");
const types_1 = require("./types");
const models_1 = require("./models");
const types_2 = require("./types");
/**
 * Base type for all response/messages. Responsible for creating new Commands to send to Elk M1 OR
 * parsing/validating a response from Elk.
 * @export
 * @class ElkMessage
 */
class ElkMessage {
    /**
     * Creates an instance of ElkMessage.
     *
     * @param {string} command - Instantiate message from a new command, i.e. command = 'zs'.
     * @param {string} response - Instantiate message from an Elk response, i.e. response = '0AZC002200CE'.
     */
    constructor(command, response) {
        const futureUse = '00';
        if (command !== null && response !== null) {
            throw new TypeError('Provide only one argument: Either command or response');
        }
        if (command !== null) {
            this.body = command.substring(2, command.length) + futureUse;
            this.type = command.substring(0, 2);
            let length = (this.type + this.body).length + 2;
            let lengthStr = length.toString(16).toUpperCase();
            if (lengthStr.length === 1) {
                lengthStr = '0' + lengthStr;
            }
            this.hexLength = lengthStr;
            this.checkSum = this.calcChecksum();
            this.message = `${this.hexLength}${this.type}${this.body}${this.checkSum}`;
        }
        else {
            let length = response.length;
            this.message = response;
            this.body = response.substring(4, length - 2);
            this.type = response.substring(2, 4);
            this.hexLength = response.substring(0, 2);
            this.checkSum = response.substring(length - 2, length);
            // If it's a message I'm listening for, validate checksum
            if (types_2.responseTypes.get(this.type) === undefined) {
                if (!this.validChecksum()) {
                    throw new TypeError(`The calculated checksum does not match the checksum on the received message: ${this.message}`);
                }
            }
        }
    }
    validChecksum() {
        return this.checkSum === this.calcChecksum();
    }
    calcChecksum() {
        const buf = new Buffer(`${this.hexLength}${this.type}${this.body}`);
        let sum = 0;
        for (let i = 0; i < buf.length; i++) {
            sum = sum + buf[i];
        }
        let chars = (((sum & 0xff) ^ 0xff) + 1) & 0xff;
        return this.hex2(chars);
    }
    /**
     * Convert to 2 byte hex value.
     *
     * @param {number} i
     * @returns
     */
    hex2(i) {
        let hex = i.toString(16).toUpperCase();
        if (hex.length === 1) {
            hex = `0${hex}`;
        }
        return hex;
    }
}
exports.ElkMessage = ElkMessage;
/**
 * Represents an Area Change Update message (AS)
 *
 * @export
 * @class ArmingStatusReport
 * @extends {ElkMessage}
 */
class ArmingStatusReport extends ElkMessage {
    constructor(response) {
        super(null, response);
        this.areas = [];
        const armStatuses = this.body.substring(0, 8).split('');
        const armUpStates = this.body.substring(8, 16).split('');
        const alarmStates = this.body.substring(16, 24).split('');
        for (let i = 0; i < 8; i++) {
            this.areas.push(new models_1.AreaReport(i + 1, types_1.armStatus.get(armStatuses[i]), types_1.armUpState.get(armUpStates[i]), types_1.alarmState.get(alarmStates[i])));
        }
    }
}
exports.ArmingStatusReport = ArmingStatusReport;
/**
 * Represents Entry/Exit Time Data (EE)
 *
 * @export
 * @class EntryExitTime
 * @extends {ElkMessage}
 */
class EntryExitTime extends ElkMessage {
    constructor(response) {
        super(null, response);
        this.areaId = +this.body.substring(0, 1);
        this.timerType = +this.body.substring(1, 2);
        this.timer1 = +this.body.substring(2, 5);
        this.timer2 = +this.body.substring(5, 8);
        this.armedState = types_1.armStatus.get(this.body.substring(8, 9));
    }
}
exports.EntryExitTime = EntryExitTime;
// export class KeypadFunctionKeypress extends ElkMessage {
//   keypadId: number;
//   functionKey: number;
//   areaChimeMode: string[] = [];
//   constructor(response: string) {
//     super(null, response);
//     this.keypadId = +this.body.substring(0, 2);
//     this.functionKey = + this.body.substring(2, 3);
//   }
// }
/**
 * Keypad Areas Report (KA).
 *
 * @export
 * @class KeypadAreasReport
 * @extends {ElkMessage}
 */
class KeypadAreasReport extends ElkMessage {
    constructor(response) {
        super(null, response);
        /**
         * Distinct areas that are assigned to keypads.
         *
         * @type {number[]}
         */
        this.areas = [];
        for (let i = 0; i < 16; i++) {
            let area = parseInt(this.body.substring(i, i + 1));
            if (area === 0) {
                continue;
            }
            if (this.areas.indexOf(area) === -1) {
                this.areas.push(area);
            }
        }
    }
}
exports.KeypadAreasReport = KeypadAreasReport;
/**
 * Represents System Log Data Update (LD)
 *
 * @export
 * @class LogDataUpdate
 * @extends {ElkMessage}
 */
class LogDataUpdate extends ElkMessage {
    constructor(response) {
        super(null, response);
        this.event = this.getEventType(+this.body.substring(0, 4));
        this.id = +this.body.substring(4, 7);
        this.areaId = +this.body.substring(7, 8);
        this.hour = this.body.substring(8, 10);
        this.minute = this.body.substring(10, 12);
        this.month = enums_1.Month[+this.body.substring(12, 14)];
        this.day = +this.body.substring(14, 16);
        this.logIndex = this.body.substring(16, 19);
        this.dayOfWeek = enums_1.WeekDay[+this.body.substring(19, 20)];
        this.year = this.body.substring(20, 22);
    }
    getEventType(event) {
        if (event >= 4001 && event <= 4208) {
            return `Zone ${event - 4000} Status: ${event === 1 ? 'violated' : 'normal'}`;
        }
        else if (event >= 5001 && event <= 5208) {
            return `Zone ${event - 4000}  Bypassed: ${event === 1 ? 'bypassed' : ''}`;
        }
        else if (event >= 6001 && event <= 6208) {
            return `Alarm Memory: ${event === 1 ? 'alarm activated' : ''}`;
        }
        else if (event >= 7001 && event <= 7208) {
            return `Output ${event - 4000}  Status: ${event === 1 ? 'On' : 'Off'}`;
        }
        else {
            return types_1.EventType.get(event.toString());
        }
    }
}
exports.LogDataUpdate = LogDataUpdate;
/**
 * Represents an Output Change Update (CC). This message is sent in response to an output
 * status change. Message contains the output number and the new state.
 * @export
 * @class OutputChangeUpdate
 * @extends {ElkMessage}
 */
class OutputChangeUpdate extends ElkMessage {
    constructor(response) {
        super(null, response);
        this.id = +this.body.substring(0, 3);
        this.state = this.body.substring(3, 4) === '1' ? 'On' : 'Off';
    }
}
exports.OutputChangeUpdate = OutputChangeUpdate;
/**
 * Represents a Control Output Status Report (CS). This message is sent in response to a
 * Control Output Status Request.
 * @export
 * @class OutputStatusReport
 * @extends {ElkMessage}
 */
class OutputStatusReport extends ElkMessage {
    constructor(response) {
        super(null, response);
        /**
         * Array of output status'. Index 0 = Output 1.
         *
         * @type {string[]}
         */
        this.outputs = [];
        for (let i = 0; i < 208; i++) {
            let status = this.body.substring(i, i + 1) === '0' ? 'Off' : 'On';
            this.outputs.push(status);
        }
    }
}
exports.OutputStatusReport = OutputStatusReport;
/**
 * Represents Temerature Reply Data (LW), in reponse to a temperature request.
 *
 * @export
 * @class TemperatureReply
 * @extends {ElkMessage}
 */
class TemperatureReply extends ElkMessage {
    constructor(response) {
        super(null, response);
        /**
         * Array of keypad temps, 1-16. Index 0 = keypad 1.
         *
         * @type {number[]}
         */
        this.keypads = [];
        /**
         * Array of zone sensor temps, 1-16. Index 0 = zone 1.
         *
         * @type {number[]}
         */
        this.zones = [];
        ;
        for (let i = 0; i <= 96; i += 3) {
            if (i < 48) {
                this.keypads.push(+this.body.substring(i, i + 3) - 40);
            }
            else if (i < 96) {
                this.zones.push(+this.body.substring(i, i + 3) - 60);
            }
        }
    }
}
exports.TemperatureReply = TemperatureReply;
/**
 * Represents a Thermostat Data Reply (TR).
 *
 * @export
 * @class ThermostatReply
 * @extends {ElkMessage}
 */
class ThermostatReply extends ElkMessage {
    constructor(response) {
        super(null, response);
        let thermNo = this.body.substring(0, 2);
        this.id = thermNo === '0' ? 'Invalid' : parseInt(thermNo).toString();
        this.mode = enums_1.ThermostatMode[this.body.substring(2, 3)];
        this.hold = !this.body.substring(3, 4);
        this.fan = this.body.substring(4, 5) === '0' ? 'Auto' : 'On';
        this.temperature = +this.body.substring(5, 7);
        this.heatSetPoint = parseFloat(this.body.substring(7, 9));
        this.coolSetPoint = parseFloat(this.body.substring(9, 11));
        let humidity = this.body.substring(11, 12);
        this.humidity = humidity === '0' ? 'No Data' : humidity;
    }
}
exports.ThermostatReply = ThermostatReply;
/**
 * Represents String Text Description (SD)
 *
 * @export
 * @class TextStringDescriptionReport
 * @extends {ElkMessage}
 */
class TextStringDescriptionReport extends ElkMessage {
    constructor(response) {
        super(null, response);
        this.descriptionType = enums_1.TextDescriptionType[parseInt(this.body.substring(0, 2))];
        this.id = parseInt(this.body.substring(2, 5));
        this.description = this.body.substring(5, this.body.length - 2).trim();
    }
}
exports.TextStringDescriptionReport = TextStringDescriptionReport;
/**
 * Represents a Task Change Update (TC). This message is sent when a task has
 * been activated.
 * @export
 * @class TaskChangeUpdate
 * @extends {ElkMessage}
 */
class TaskChangeUpdate extends ElkMessage {
    constructor(response) {
        super(null, response);
        this.taskNumber = +this.body.substring(0, 3);
    }
}
exports.TaskChangeUpdate = TaskChangeUpdate;
/**
 * Represents a Zone Bypass Reply (ZB)
 *
 * @export
 * @class ZoneBypass
 * @extends {ElkMessage}
 */
class ZoneBypass extends ElkMessage {
    constructor(response) {
        super(null, response);
        this.id = +this.body.substring(0, 3);
        this.bypassed = this.body.substring(3, 4) === '1' ? true : false;
    }
}
exports.ZoneBypass = ZoneBypass;
/**
 * Represents a Zone Definition Report (ZD)
 *
 * @export
 * @class ZoneDefinitionReport
 * @extends {ElkMessage}
 */
class ZoneDefinitionReport extends ElkMessage {
    constructor(response) {
        super(null, response);
        this.zones = [];
        for (let i = 0; i < 208; i++) {
            let def = this.body.substring(i, i + 1);
            this.zones.push(new models_1.ZoneDefinition(i + 1, types_1.zoneDefinition.get(def)));
        }
    }
}
exports.ZoneDefinitionReport = ZoneDefinitionReport;
/**
 * Represents a Zone Voltage Report (ZV)
 *
 * @export
 * @class ZoneVoltageReport
 * @extends {ElkMessage}
 */
class ZoneVoltageReport extends ElkMessage {
    constructor(response) {
        super(null, response);
        this.id = +this.body.substring(0, 3);
        this.voltage = +this.body.substring(3, 6) / 10;
    }
}
exports.ZoneVoltageReport = ZoneVoltageReport;
/**
 * Represents a Zone Change Update message (ZC).
 *
 * @export
 * @class ZoneChangeUpdate
 * @extends {ElkMessage}
 */
class ZoneChangeUpdate extends ElkMessage {
    constructor(response) {
        super(null, response);
        this.id = +this.body.substring(0, 3);
        // Parse Hex value into physical & logical state/status
        let status = parseInt(this.body.substring(3, 4), 16);
        this.physicalStatus = enums_1.PhysicalStatus[status & 0x03];
        this.logicalState = enums_1.LogicalState[status >> 2];
    }
}
exports.ZoneChangeUpdate = ZoneChangeUpdate;
/**
 * Represents Zone Status Report (ZS)
 *
 * @export
 * @class ZoneStatusReport
 */
class ZoneStatusReport extends ElkMessage {
    constructor(response) {
        super(null, response);
        this.zones = [];
        // Elk M1 supports up to 208 zones
        for (let z = 1; z <= 208; z++) {
            var status = parseInt(this.body.substring(z - 1, z), 16);
            this.zones.push({
                id: z,
                physicalStatus: enums_1.PhysicalStatus[status & 0x03],
                logicalState: enums_1.LogicalState[status >> 2]
            });
        }
    }
}
exports.ZoneStatusReport = ZoneStatusReport;
/**
 * Represents Zone Partition Report (ZP)
 *
 * @export
 * @class ZonePartitionReport
 * @extends {ElkMessage}
 */
class ZonePartitionReport extends ElkMessage {
    constructor(response) {
        super(null, response);
        this.zones = [];
        // Elk M1 supports up to 208 zones
        for (let z = 1; z <= 208; z++) {
            this.zones.push({
                id: z,
                partition: +this.body.substring(z - 1, z)
            });
        }
    }
}
exports.ZonePartitionReport = ZonePartitionReport;
